@@ -43,9 +43,11 @@
 #define MARGIN_X 2
 #define MARGIN_Y 2
 
-GtkWidget *window, *main_scrolled_window, *fullscreen_window, *fullscreen_toggle_popup, *vbox2;
+#define TOOLBAR_TIMEOUT 2500
+
+GtkWidget *window, *main_scrolled_window, *fullscreen_window, *fullscreen_toggle_popup, *fullscreen_toolbar_popup, *vbox2;
 GtkWidget *dirbrowser_window;
-GtkWidget *view_widget, *image_widget;
+GtkWidget *view_widget = NULL, *image_widget;
 GdkPixbuf *image_pixbuf, *scaled_image_pixbuf = NULL;
 GtkWidget *image_event_box;
 GtkWidget *loading_toolbar, *tools_toolbar;
@@ -61,11 +63,16 @@ gint fullscreen_state = 0;
 gboolean confine_pointer_to_window;
 
 guint slideshow_timer = 0;
+guint fullscreen_toolbar_timer = 0;
+guint fullscreen_pointer_timer = 0;
 
 guint x_start, y_start, x_max, y_max;
+gint pointer_x, pointer_y = 0;
 double xadj_start, yadj_start;
 guint zoom_timer_id;
 
+static void toggle_fullscreen ();
+
 struct gpe_icon my_icons[] = {
   { "open", "open" },
   { "left", "left" },
@@ -79,8 +86,8 @@ struct gpe_icon my_icons[] = {
   { "pan", "gallery/pan" },
   { "rotate", "gallery/rotate" },
   { "sharpen", "gallery/sharpen" },
-  { "list_view", "gallery/list_view" },
-  { "icon_view", "gallery/icon_view" },
+  { "list_view", "list-view" },
+  { "icon_view", "icon-view" },
   { "blur", "gallery/blur" },
   { "ok", "ok" },
   { "cancel", "cancel" },
@@ -122,6 +129,42 @@ update_window_title (void)
   gtk_window_set_title (GTK_WINDOW (window), window_title);
 }
 
+static gboolean
+hide_fullscreen_toolbar ()
+{
+  gtk_widget_hide (fullscreen_toolbar_popup);
+
+  return FALSE;
+}
+
+static gboolean
+check_fullscreen_pointer ()
+{
+  gint x, y;
+  
+  gdk_display_get_pointer (gdk_display_get_default (), NULL, &x, &y, NULL);
+  
+  if ((x != pointer_x) || (y != pointer_y))
+  {
+    if (fullscreen_toolbar_timer != 0)
+    {
+      g_source_remove (fullscreen_toolbar_timer);
+      fullscreen_toolbar_timer = 0;
+    }
+    gtk_widget_show_all (fullscreen_toolbar_popup);
+    gtk_widget_set_uposition (fullscreen_toolbar_popup, 0, fullscreen_window->allocation.height - fullscreen_toolbar_popup->allocation.height);
+  }
+  else
+  {
+    if (fullscreen_toolbar_timer == 0)
+      fullscreen_toolbar_timer = g_timeout_add (TOOLBAR_TIMEOUT, hide_fullscreen_toolbar, NULL);
+  }
+  
+  gdk_display_get_pointer (gdk_display_get_default (), NULL, &pointer_x, &pointer_y, NULL);
+  
+  return TRUE;
+}
+
 static void
 pan_button_down (GtkWidget *w, GdkEventButton *b, GtkWidget *scrolled_window)
 {
@@ -186,7 +229,8 @@ show_image (GtkWidget *widget, GList *image_filename)
   float width_ratio, height_ratio;
   float scale_width_ratio, scale_height_ratio;
 
-  gtk_widget_hide (view_widget);
+  if (view_widget)
+  	gtk_widget_hide (view_widget);
 
   image_filenames = image_filename;
   image_pixbuf = gdk_pixbuf_new_from_file ((gchar *) image_filename->data, NULL);
@@ -233,53 +277,6 @@ open_from_file (gchar *filename)
   show_image (NULL, image_filenames);
 }
 
-static void
-toggle_fullscreen ()
-{
-  GtkWidget *scrolled_window, *image_widget, *image_event_box, *close_button;
-
-  if (fullscreen_state == 1)
-  {
-    gtk_widget_destroy (fullscreen_window);
-    gtk_widget_destroy (fullscreen_toggle_popup);
-    fullscreen_state = 0;
-  }
-  else if (fullscreen_state == 0)
-  {
-    fullscreen_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-
-    scrolled_window = gtk_scrolled_window_new (NULL, NULL);
-    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window), GTK_POLICY_NEVER, GTK_POLICY_NEVER);
-
-    image_widget = gtk_image_new_from_pixbuf (scaled_image_pixbuf);
-    image_event_box = gtk_event_box_new ();
-    gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrolled_window), image_event_box);
-
-    gtk_signal_connect (GTK_OBJECT (image_event_box), "button-press-event", GTK_SIGNAL_FUNC (button_down), scrolled_window);
-    gtk_signal_connect (GTK_OBJECT (image_event_box), "button-release-event", GTK_SIGNAL_FUNC (button_up), NULL);
-    gtk_signal_connect (GTK_OBJECT (image_event_box), "motion-notify-event", GTK_SIGNAL_FUNC (motion_notify), scrolled_window);
-
-    gtk_widget_add_events (GTK_WIDGET (image_event_box), GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK);
-
-    gtk_container_add (GTK_CONTAINER (image_event_box), image_widget);
-    gtk_container_add (GTK_CONTAINER (fullscreen_window), scrolled_window);
-
-    gtk_widget_set_size_request (fullscreen_window, gdk_screen_width (), gdk_screen_height ());
-    gtk_widget_show_all (fullscreen_window);
-
-    gdk_window_fullscreen (GDK_WINDOW (fullscreen_window->window));
-
-    fullscreen_toggle_popup = gtk_window_new (GTK_WINDOW_POPUP);
-    close_button = gpe_button_new_from_stock (GTK_STOCK_CLOSE, GPE_BUTTON_TYPE_ICON);
-    g_signal_connect (G_OBJECT (close_button), "clicked",
-		      G_CALLBACK (toggle_fullscreen), NULL);
-    gtk_container_add (GTK_CONTAINER (fullscreen_toggle_popup), close_button);
-    gtk_widget_show_all (fullscreen_toggle_popup);
-
-    fullscreen_state = 1;
-  }
-}
-
 static gboolean
 image_zoom_hyper ()
 {
@@ -365,7 +362,6 @@ image_zoom_out ()
   g_object_unref (image_pixbuf);
 
   zoom_timer_id = gtk_timeout_add (1000, image_zoom_hyper, scaled_image_pixbuf);
-
 }
 
 void
@@ -387,9 +383,16 @@ image_zoom_fit ()
   g_object_unref (scaled_image_pixbuf);
   image_pixbuf = gdk_pixbuf_new_from_file ((gchar *) image_filenames->data, NULL);
 
-  widget_width = main_scrolled_window->allocation.width - 5;
-  widget_height = main_scrolled_window->allocation.height - 5;
-
+  if (fullscreen_state == 1)
+  {
+    widget_width = main_scrolled_window->allocation.width - 5;
+    widget_height = main_scrolled_window->allocation.height - 5;
+  }
+  else
+ {
+    widget_width = vbox2->allocation.width - 5;
+    widget_height = vbox2->allocation.height - 5;
+  }
   width = gdk_pixbuf_get_width (GDK_PIXBUF (image_pixbuf));
   height = gdk_pixbuf_get_height (GDK_PIXBUF (image_pixbuf));
 
@@ -452,6 +455,9 @@ render_list_view_button_press_event (GtkWidget *widget, GdkEventButton *event, G
   if (event->button != 1)
     return FALSE;
 
+  if (loaded_images == NULL)
+    return FALSE;
+
   this_item = loaded_images;
 
   while (current_y <= target_y)
@@ -487,6 +493,9 @@ render_list_view_button_release_event (GtkWidget *widget, GdkEventButton *event,
   if (event->button != 1)
     return FALSE;
 
+  if (loaded_images == NULL)
+    return FALSE;
+
   this_item = loaded_images;
 
   while (current_y <= target_y)
@@ -640,7 +649,7 @@ render_list_view ()
 	g_object_unref (list_item->pixbuf);
 	list_item->pixbuf = pixbuf;
       }
-    }
+    
 
     list_item->pango_layout = gtk_widget_create_pango_layout (drawing_area, NULL);
     pango_layout_set_width (list_item->pango_layout, -1);
@@ -657,6 +666,7 @@ render_list_view ()
 
     while (gtk_events_pending ())
       gtk_main_iteration ();
+   }
   }
 
   g_signal_connect (G_OBJECT (drawing_area), "expose_event", 
@@ -737,6 +747,29 @@ render_view (GtkWidget *parent, gint view)
   gtk_widget_hide (loading_toolbar);
 }
 
+static
+gboolean is_image_file(const gchar* filename)
+{
+  gchar *fn = g_utf8_strdown(filename,-1);
+  gboolean result = FALSE;
+
+  if (strstr (fn, ".png")) 
+	  result = TRUE;
+  if (strstr (fn, ".jpg") || strstr (filename, ".jpeg"))
+	  result = TRUE;
+  if (strstr (fn, ".gif"))
+	  result = TRUE;
+  if (strstr (fn, ".ico"))
+	  result = TRUE;
+  if (strstr (fn, ".bmp"))
+	  result = TRUE;
+  if (strstr (fn, ".xpm"))
+	  result = TRUE;
+  g_free(fn);
+  
+  return result;
+}
+
 static void
 add_directory (gchar *directory)
 {
@@ -768,7 +801,7 @@ add_directory (gchar *directory)
 
         if (stat (filename, &s) == 0)
         {
-	      if (strstr (filename, ".png") || strstr (filename, ".jpg") || strstr (filename, ".jpeg"))
+	      if (is_image_file(filename))
 	      {
 	        image_filenames = g_list_append (image_filenames, (gpointer) filename);
           }
@@ -824,7 +857,8 @@ start_slideshow (GtkWidget *widget, GtkWidget *spin_button)
     image_filenames = g_list_first (image_filenames);
     show_image (NULL, image_filenames);
     slideshow_timer = gtk_timeout_add (gtk_spin_button_get_value (GTK_SPIN_BUTTON (spin_button)) * 1000, next_image, NULL);
-  }
+    toggle_fullscreen ();
+    }
   else
   {
     stop_slideshow ();
@@ -895,6 +929,122 @@ show_new_slideshow (void)
   gtk_widget_show (slideshow_dialog);
 }
 
+static void
+toggle_fullscreen ()
+{
+  GtkWidget *scrolled_window, *image_widget, *image_event_box, *close_button, *fullscreen_toolbar, *toolbar_icon;
+  GdkPixbuf *p;
+
+  if (fullscreen_state == 1)
+  {
+    //gtk_widget_hide (main_scrolled_window);
+    hide_image ();
+    gtk_widget_reparent (main_scrolled_window, vbox2);
+    show_image (NULL, image_filenames);
+    //gtk_widget_realize (main_scrolled_window);
+    //gtk_widget_show (main_scrolled_window);
+
+    g_source_remove (fullscreen_toolbar_timer);
+    g_source_remove (fullscreen_pointer_timer);
+    
+    gtk_widget_destroy (fullscreen_window);
+    gtk_widget_destroy (fullscreen_toggle_popup);
+    gtk_widget_destroy (fullscreen_toolbar_popup);
+
+    fullscreen_state = 0;
+    fullscreen_toolbar_timer = 0;
+    fullscreen_pointer_timer = 0;
+
+    image_zoom_fit ();
+  }
+  else if (fullscreen_state == 0)
+  {
+    fullscreen_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+
+    scrolled_window = gtk_scrolled_window_new (NULL, NULL);
+    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window), GTK_POLICY_NEVER, GTK_POLICY_NEVER);
+
+    image_widget = gtk_image_new_from_pixbuf (scaled_image_pixbuf);
+    image_event_box = gtk_event_box_new ();
+    gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrolled_window), image_event_box);
+
+    gtk_signal_connect (GTK_OBJECT (image_event_box), "button-press-event", GTK_SIGNAL_FUNC (button_down), scrolled_window);
+    gtk_signal_connect (GTK_OBJECT (image_event_box), "button-release-event", GTK_SIGNAL_FUNC (button_up), NULL);
+    gtk_signal_connect (GTK_OBJECT (image_event_box), "motion-notify-event", GTK_SIGNAL_FUNC (motion_notify), scrolled_window);
+
+    gtk_widget_add_events (GTK_WIDGET (image_event_box), GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK);
+
+    gtk_container_add (GTK_CONTAINER (image_event_box), image_widget);
+    //gtk_container_add (GTK_CONTAINER (fullscreen_window), scrolled_window);
+    gtk_widget_reparent (main_scrolled_window, fullscreen_window);
+
+    gtk_widget_set_size_request (fullscreen_window, gdk_screen_width (), gdk_screen_height ());
+    gtk_widget_show_all (fullscreen_window);
+
+    gdk_window_fullscreen (GDK_WINDOW (fullscreen_window->window));
+
+    fullscreen_toggle_popup = gtk_window_new (GTK_WINDOW_POPUP);
+    close_button = gpe_button_new_from_stock (GTK_STOCK_CLOSE, GPE_BUTTON_TYPE_ICON);
+    g_signal_connect (G_OBJECT (close_button), "clicked",
+		      G_CALLBACK (toggle_fullscreen), NULL);
+    gtk_container_add (GTK_CONTAINER (fullscreen_toggle_popup), close_button);
+    gtk_widget_show_all (fullscreen_toggle_popup);
+
+  fullscreen_toolbar_popup = gtk_window_new (GTK_WINDOW_POPUP);
+
+  fullscreen_toolbar = gtk_toolbar_new ();
+  gtk_toolbar_set_orientation (GTK_TOOLBAR (fullscreen_toolbar), GTK_ORIENTATION_HORIZONTAL);
+
+  toolbar_icon = gtk_image_new_from_stock (GTK_STOCK_GO_BACK , GTK_ICON_SIZE_SMALL_TOOLBAR);
+  gtk_toolbar_append_item (GTK_TOOLBAR (fullscreen_toolbar), _("Previous"), 
+			   _("Previous image"), _("Previous image"), toolbar_icon, GTK_SIGNAL_FUNC (previous_image), NULL);
+
+  toolbar_icon = gtk_image_new_from_stock (GTK_STOCK_GO_FORWARD, GTK_ICON_SIZE_SMALL_TOOLBAR);
+  gtk_toolbar_append_item (GTK_TOOLBAR (fullscreen_toolbar), _("Next"), 
+			   _("Next image"), _("Next image"), toolbar_icon, GTK_SIGNAL_FUNC (next_image), NULL);
+
+  gtk_toolbar_append_space (GTK_TOOLBAR (fullscreen_toolbar));
+
+  p = gpe_find_icon ("zoom_in");
+  toolbar_icon = gtk_image_new_from_pixbuf (p);
+  gtk_toolbar_append_item (GTK_TOOLBAR (fullscreen_toolbar), _("Zoom In"), 
+			   _("Zoom In"), _("Zoom In"), toolbar_icon, image_zoom_in, NULL);
+
+  p = gpe_find_icon ("zoom_out");
+  toolbar_icon = gtk_image_new_from_pixbuf (p);
+  gtk_toolbar_append_item (GTK_TOOLBAR (fullscreen_toolbar), _("Zoom Out"), 
+			   _("Zoom Out"), _("Zoom Out"), toolbar_icon, image_zoom_out, NULL);
+
+  p = gpe_find_icon ("zoom_1");
+  toolbar_icon = gtk_image_new_from_pixbuf (p);
+  gtk_toolbar_append_item (GTK_TOOLBAR (fullscreen_toolbar), _("Zoom 1:1"), 
+			   _("Zoom 1:1"), _("Zoom 1:1"), toolbar_icon, image_zoom_1, NULL);
+
+  p = gpe_find_icon ("zoom_fit");
+  toolbar_icon = gtk_image_new_from_pixbuf (p);
+  gtk_toolbar_append_item (GTK_TOOLBAR (fullscreen_toolbar), _("Zoom to Fit"), 
+			   _("Zoom to Fit"), _("Zoom to Fit"), toolbar_icon, image_zoom_fit, NULL);
+
+  gtk_toolbar_append_space (GTK_TOOLBAR (fullscreen_toolbar));
+
+  p = gpe_find_icon ("rotate");
+  toolbar_icon = gtk_image_new_from_pixbuf (p);
+  gtk_toolbar_append_item (GTK_TOOLBAR (fullscreen_toolbar), _("Rotate"), 
+			   _("Toggle Rotate"), _("Toggle Rotate"), toolbar_icon, image_rotate, NULL);
+
+    gtk_container_add (GTK_CONTAINER (fullscreen_toolbar_popup), fullscreen_toolbar);
+    gtk_widget_show_all (fullscreen_toolbar_popup);
+    gtk_widget_grab_focus (toolbar_icon);
+    fullscreen_pointer_timer = g_timeout_add (200, check_fullscreen_pointer, NULL);
+    gtk_widget_set_uposition (fullscreen_toolbar_popup, 0, fullscreen_window->allocation.height - fullscreen_toolbar_popup->allocation.height);
+
+    fullscreen_state = 1;
+
+    
+    image_zoom_fit ();
+  }
+}
+
 static void
 show_dirbrowser (void)
 {
